import requests
import json
import sys, os
sys.path.append('..')
from .. import config

unique_database = str(config.auth[0])
def getResponse(input_message):
    url = 'http://facerecog.io:12000/chatbot/%s' % unique_database
    payload = {"message":input_message}
    r = requests.post(url, json=payload)
    return r.text



#! /usr/bin/env python
"""
    This is the server starter.

    It has all Yowsup core layers to handle the connection, encryption and media handling.
    https://github.com/tgalal/yowsup/wiki/Yowsup-2.0-Architecture

    The top layer of the stack is the RouteLayer (from route.py).
"""
import logging, time
from yowsup.layers import YowLayerEvent, YowParallelLayer
from yowsup.layers.auth import AuthError
from yowsup.layers.network import YowNetworkLayer
from yowsup.stacks.yowstack import YowStackBuilder

from wikimon_bot.layers.notifications.notification_layer import NotificationsLayer
from wikimon_bot.router import RouteLayer

import argparse

parser = argparse.ArgumentParser(description="Run wikimon")
parser.add_argument("-p", "--phone", metavar="phone number", required=True, type=str, help="Phone number used for registration")
parser.add_argument("-pw", "--password", metavar="password", required=True, type=str, help="14-20 digit password")
args = parser.parse_args()
wikimonauth = (args.phone, args.password)

class YowsupEchoStack(object):
    def __init__(self, credentials):
        "Creates the stacks of the Yowsup Server,"
        self.credentials = credentials
        stack_builder = YowStackBuilder().pushDefaultLayers(True)

        # on the top stack, the two layers that controls the bot and respond to messages and notifications
        # see both of classes for more documentation
        stack_builder.push(YowParallelLayer([RouteLayer, NotificationsLayer]))
        self.stack = stack_builder.build()
        self.stack.setCredentials(credentials)

    def start(self):
        "Starts the connection with Whatsapp servers,"
        self.stack.broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_CONNECT))
        try:
            logging.info("#" * 50)
            logging.info("\tServer started. Phone number: %s" % self.credentials[0])
            logging.info("#" * 50)
            self.stack.loop(timeout=0.5, discrete=0.5)
        except AuthError as e:
            logging.exception("Authentication Error: %s" % e.message)
        except Exception as e:
            logging.exception("Unexpected Exception: %s" % e.message)


if __name__ == "__main__":
    import sys
    import config

    logging.basicConfig(stream=sys.stdout, level=config.logging_level, format=config.log_format)
    server = YowsupEchoStack(wikimonauth)
    while True:
        # In case of disconnect, keeps connecting...
        server.start()
        logging.info("Restarting..")



"""
    BingViews:
    /i(mage) <term>

"""
from ..utils import media_sender
import requests, urllib
from .. import config

class BingViews():
    def __init__(self, interface_layer):
        self.image_sender = media_sender.ImageSender(interface_layer)
        self.routes = [
            ("/i(mage)?\s(?P<term>[^$]+)$", self.bing_image_search)
        ]

    def bing_image_search(self, message, match):
        req = requests.get("https://api.datamarket.azure.com/Bing/Search/v1/Image?Query=%27{}%27&$format=json&$top=1".format(match.group("term")), auth=("",config.bing_api))
        image_url = urllib.unquote(req.json()['d']['results'][0]['MediaUrl'].encode('utf-8'))
        self.image_sender.send_by_url(jid=message.getFrom(), file_url=image_url)



from .. import config
import random
from .. utils import session 
from yowsup.layers.protocol_messages.protocolentities import TextMessageProtocolEntity

"""
    In this Views All methods will have access to a map like object, called session_db, to store some history from users.
    session_db is a Object from SessionDB, check there for more info!
"""


class QuizView:
    def __init__(self, interface_layer):
        self.interface_layer = interface_layer
        self.session_db = session.SessionDB(config.session_db_path, "quiz")

        # One route to start a new quiz, one to answer.
        self.routes = [
            ("^/quiz\s?$", self.quiz),
            (r"^(?P<quiz_answer>\d{1})\s?$", self.quiz_answer),  # 0-9
        ]

    def quiz(self, message, match):
        # Gets a random quiz and store in the sender' session
        quiz = self._get_quiz()
        self.session_db.set(message.getFrom(), quiz)
        return TextMessageProtocolEntity(self._get_quiz_text(quiz), to=message.getFrom())

    def quiz_answer(self, message, match):
        # if there is a quiz stored on the sender' session, this is an answer, otherwise ignore it
        quiz = self.session_db.get(message.getFrom())
        if quiz:
            self.session_db.set(message.getFrom(), None)
            ans = int(match.group("quiz_answer"))
            if ans == quiz["correct_alternative"]:
                return TextMessageProtocolEntity("Correct!", to=message.getFrom())
            else:
                msg_wrong = "Wrong. Correct answer was: %s" % quiz["correct"]
                return TextMessageProtocolEntity(msg_wrong, to=message.getFrom())

    def _get_quiz_text(self, quiz):
        ans = "\n".join(["(%s) %s" % (k, v) for k, v in quiz["alternatives"].iteritems()])
        return "\n".join([quiz["question"], ans])

    def _get_quiz(self):
        """
        Returns a random quiz like this:
        {
            "question":"How much is 1 + 2 + 3?",
           "alternatives": {
               1:6,
               2:3,
               3:"potato",
               ...
               "d":7
           },
            "correct_alternative": 1
        }
        """
        values = random.sample(range(1, 100), random.randint(2, 5))
        sum_values = sum(values)
        ans = [sum_values + int(random.uniform(-20, 20)) for i in range(3)]
        ans.append(sum_values)
        random.shuffle(ans)

        alternatives = {}
        for i, a in enumerate(ans):
            alternatives[i + 1] = a
        correct_alternative = ans.index(sum_values) + 1

        return {
            "question": "How much is %s?" % " + ".join(str(v) for v in values),
            "alternatives": alternatives,
            "correct_alternative": correct_alternative
        }



"""
    GoogleViews:
    /s(earch) <term>
    /i(mage) <term>
    youtube urls

"""
from ..utils import media_sender
import requests, urllib


class GoogleViews():
    def __init__(self, interface_layer):
        self.image_sender = media_sender.ImageSender(interface_layer)
        self.video_sender = media_sender.VideoSender(interface_layer)
        self.yt_sender = media_sender.YoutubeSender(interface_layer)
        self.url_print_sender = media_sender.UrlPrintSender(interface_layer)
        self.routes = [
            (".*https?:\/\/(?:www\.|m\.)?youtu(?:be.com\/watch\?v=|\.be/)(?P<video_id>[\w-]+)(&\S*)?$",
             self.send_yt_video),
            ("/s(earch)?\s(?P<term>[^$]+)$", self.google_search),
        ]

    def send_yt_video(self, message, match):
        self.yt_sender.send_by_url(jid=message.getFrom(), file_url=match.group("video_id"))

    def google_search(self, message, match):
        req = requests.get("http://ajax.googleapis.com/ajax/services/search/web?v=1.0&q=%s" % match.group("term"))
        page_url = urllib.unquote(req.json()["responseData"]["results"][0]["url"])
        self.url_print_sender.send_by_url(jid=message.getFrom(), file_url=page_url)



"""
    Credentials to connect on Whatsapp Servers.
    (phone number, whatsapp key)
    To extract key use the yowsup-cli (using a python venv with yowsup installed):
    > yowsup-cli registration -C <CountryCode> -r sms -p <Phone Number with Country Code>
    ex.:
    yowsup-cli registration -C 55 -r sms -p 554899998888
    Then whatsapp will send a key via sms to the phone.
    Get that key then run:
    > yowsup-cli registration -C 55 -R <sms-key> -p 554899998888
    status: ok
    kind: free
    > pw: njH+QGBqGXXXXXXXOFa+Wth5riM=
    price: US$0.99
    price_expiration: 1444272405
    currency: USD
    cost: 0.99
    > login: 554899998888
    type: existing
    expiration: 1472404969
    Now just get the login and pw, and replace bellow.
    If have any problem on registragion with yowsup, you can always register with
    https://github.com/mgp25/Chat-API/wiki/Extracting-password-from-device#using-apk
"""
import os, logging

defaults = {
    "WHATSAPP_LOGIN": "abc",
    "WHATSAPP_PW": "123",
    "WHATSAPP_ADMIN": "",
    "BING_API_KEY": ""
}

auth = (os.environ.get('WHATSAPP_LOGIN', defaults['WHATSAPP_LOGIN']),
        os.environ.get('WHATSAPP_PW', defaults['WHATSAPP_PW']))

# If filter_groups is True, the bot only stays
# at groups that there is at least one admin on it.
# Otherwise will leave instantly if added.
filter_groups = True
admins = [os.environ.get('WHATSAPP_ADMIN', defaults['WHATSAPP_ADMIN']), ]

# Bing API for image search
bing_api_key = os.environ.get('BING_API_KEY', defaults['WHATSAPP_ADMIN'])

# Path to download the media requests
# (audio recordings, printscreens, media and youtube videos)
media_storage_path = "/tmp/"

# Session shelve db path
session_db_path = "/tmp/sessions.db"

# Logging configuration.
# By default only logs the command messages.
# If logging_level set to logging.DEBUG, yowsup will log every protocoll message exchange with server.
log_format = '_%(filename)s_\t[%(levelname)s][%(asctime)-15s] %(message)s'
logging_level = logging.INFO



"""
    The Route Layer.
    Here the message is routed to its proper view.
    The routes are defined with regular expressions and callback functions (just like any web framework).
"""
import threading
import re
import logging

from yowsup.layers.interface import YowInterfaceLayer, ProtocolEntityCallback

from views import basic_views
from views.media import MediaViews
from views.super_views import SuperViews
from views.group_admin import GroupAdminViews
from views.google import GoogleViews
from views.bing import BingViews
from views.quiz import QuizView


# Basic regex routes
routes = [("^/ping", basic_views.ping),
          ("^/e(cho)?\s(?P<echo_message>[^$]+)$", basic_views.echo),
	  ("^((?!http|/[a-zA-Z]).)*$", basic_views.superbot)]


class RouteLayer(YowInterfaceLayer):
    def __init__(self):
        """
            The definition of routes and views (callbacks)!

            For the simple message handling, just calls the callback function, and expects a message entity to return.
            For more complex handling, like asynchronous file upload and sending, it creates a object passing 'self',
            so the callback can access the 'self.toLower' method
        """
        super(RouteLayer, self).__init__()

        # Google views to handle tts, search and youtube
        routes.extend(GoogleViews(self).routes)

        # Bing views to handle image search
        routes.extend(BingViews(self).routes)

        # Media views to handle url print screen and media download
        routes.extend(MediaViews(self).routes)

        # adds super fun views
        routes.extend(SuperViews(self).routes)

        # adds quiz views
        routes.extend(QuizView(self).routes)

        # group admin views disabled by default.
        # read the issue on: https://github.com/joaoricardo000/whatsapp-bot-seed/issues/4
        # enable on your own risk!
        # routes.extend(GroupAdminViews(self).routes)

        self.views = [(re.compile(pattern), callback) for pattern, callback in routes]

    def route(self, message):
        "Get the text from message and tests on every route for a match"
        text = message.getBody()
        for route, callback in self.views:
            match = route.match(text)
            if match:  # in case of regex match, the callback is called, passing the message and the match object
                threading.Thread(target=self.handle_callback, args=(callback, message, match)).start()
                break

    def handle_callback(self, callback, message, match):
        try:
            # log message request
            if (message.isGroupMessage()):
                logging.info("(GROUP)[%s]-[%s]\t%s" % (message.getParticipant(), message.getFrom(), message.getBody()))
            else:
                logging.info("(PVT)[%s]\t%s" % (message.getFrom(), message.getBody()))
            # execute callback request
            data = callback(message, match)
            if data: self.toLower(data)  # if callback returns a message entity, sends it.
        except Exception as e:
            logging.exception("Error routing message: %s\n%s" % (message.getBody(), message))

    @ProtocolEntityCallback("message")
    def on_message(self, message):
        "Executes on every received message"
        self.toLower(message.ack())  # Auto ack
        self.toLower(message.ack(True))  # Auto ack (double blue check symbol)
        # Routing only text type messages, for now ignoring other types. (media, audio, location...)
        if message.getType() == 'text':
            self.route(message)

    @ProtocolEntityCallback("receipt")
    def on_receipt(self, entity):
        "Auto ack for every message receipt confirmation"
        self.toLower(entity.ack())



"""
    This layers only respond to whatsapp server notifications.
    Notifications can be a inform that the bot was added in a group, or someone was added to a group, or anything like that.
    Check yowsup.layers.protocol_groups.protocolentities for all notification protocol objects in Yowsup.

    Is this implementation we use the CreateGroupsNotificationProtocolEntity to check if the bot was
    added in a group, and leave if it is not allowed.
"""
from ... import config
from yowsup.layers.interface import YowInterfaceLayer, ProtocolEntityCallback
from yowsup.layers.protocol_groups.protocolentities.iq_groups_leave import LeaveGroupsIqProtocolEntity
from yowsup.layers.protocol_groups.protocolentities.iq_result_groups_list import ListGroupsResultIqProtocolEntity
from yowsup.layers.protocol_groups.protocolentities.notification_groups_add import AddGroupsNotificationProtocolEntity
from yowsup.layers.protocol_groups.protocolentities.notification_groups_create import \
    CreateGroupsNotificationProtocolEntity
from yowsup.layers.protocol_groups.protocolentities.notification_groups_remove import \
    RemoveGroupsNotificationProtocolEntity


class NotificationsLayer(YowInterfaceLayer):
    def __init__(self):
        super(NotificationsLayer, self).__init__()

    @ProtocolEntityCallback("notification")
    def onNotification(self, notification):
        """
            Reacts to any notification received
        """
        self.toLower(notification.ack())
        if isinstance(notification, CreateGroupsNotificationProtocolEntity):  # added on new group
            self.on_created_group(notification)
        elif isinstance(notification, ListGroupsResultIqProtocolEntity):  # result of a query of all groups
            self.on_groups_list(notification)
            # elif isinstance(notification, RemoveGroupsNotificationProtocolEntity):
            #     pass
            # elif isinstance(notification, AddGroupsNotificationProtocolEntity):
            #     pass

    def on_groups_list(self, listGroupResultEntity):
        groups = listGroupResultEntity.getGroups()
        for g in groups:
            if not self.is_allowed_on_group(g):
                self.leave_group(g)

    def on_created_group(self, createGroupsNotificationProtocolEntity):
        group_id = createGroupsNotificationProtocolEntity.getGroupId() + "@g.us"
        if self.is_allowed_on_group(createGroupsNotificationProtocolEntity):
            # this is a good place to a "Hello Group" message
            pass
        else:
            self.toLower(LeaveGroupsIqProtocolEntity(group_id))

    def is_allowed_on_group(self, group_entity):
        if config.filter_groups:
            isAllowed = False
            for jid, isAdmin in group_entity.getParticipants().iteritems():
                if jid.split("@")[0] in config.admins:
                    isAllowed = True
                    break
        else:
            isAllowed = True
        return isAllowed

    def leave_group(self, group_entity):
        group_id = group_entity.getGroupId() + "@g.us"
        self.toLower(LeaveGroupsIqProtocolEntity(group_id))


"""
##  Removed

notification
<class 'yowsup.layers.protocol_groups.protocolentities.notification_groups_remove.RemoveGroupsNotificationProtocolEntity'>
{'_type': 'w:gp2', 'timestamp': 1443379571, '_participant': '554896270570@s.whatsapp.net', 'participants': {'554898607439@s.whatsapp.net': None}, 'tag': 'notification', 'notify': 'Jo\xc3\xa3o Ricardo', '_from': '554896270570-1443068696@g.us', '_id': '554896270570-1443068696@g.us', 'offline': False, 'subject': 'kick admin'}

##  Added

notification
<class 'yowsup.layers.protocol_groups.protocolentities.notification_groups_create.CreateGroupsNotificationProtocolEntity'>
{'subjectTime': 1443068696, '_type': 'w:gp2', 'createType': None, 'timestamp': 1443379650, 'creatorJid': '554896270570@s.whatsapp.net', '_participant': '554896270570@s.whatsapp.net', 'groupId': '554896270570-1443068696', 'participants': {'554896270570@s.whatsapp.net': 'admin', '554898607439@s.whatsapp.net': None}, 'tag': 'notification', 'notify': 'Jo\xc3\xa3o Ricardo', 'subjectOwnerJid': '554896270570@s.whatsapp.net', '_from': '554896270570-1443068696@g.us', '_id': '554896270570-1443068696@g.us', 'creationTimestamp': 1443068696, 'offline': False, 'subject': 'kick admin'}

##  Member added

notification
<class 'yowsup.layers.protocol_groups.protocolentities.notification_groups_add.AddGroupsNotificationProtocolEntity'>
{'_type': 'w:gp2', 'timestamp': 1443379735, '_participant': '554896270570@s.whatsapp.net', 'participants': ['554898050168@s.whatsapp.net'], 'tag': 'notification', 'notify': 'Jo\xc3\xa3o Ricardo', '_from': '554896270570-1443068696@g.us', '_id': '554896270570-1443068696@g.us', 'offline': False}

## Member removed

notification
<class 'yowsup.layers.protocol_groups.protocolentities.notification_groups_remove.RemoveGroupsNotificationProtocolEntity'>
{'_type': 'w:gp2', 'timestamp': 1443380969, '_participant': '554896270570@s.whatsapp.net', 'participants': {'554899914159@s.whatsapp.net': None}, 'tag': 'notification', 'notify': 'Jo\xc3\xa3o Ricardo', '_from': '554896270570-1443068696@g.us', '_id': '554896270570-1443068696@g.us', 'offline': False, 'subject': 'kick admin'}
"""



"""
    Media download request views.

    Handles the media url messages with utilities classes for it.
"""
import hashlib
from .. import config
import subprocess
from ..utils import media_sender


class MediaViews():
    def __init__(self, interface_layer):
        """
            Creates the regex routes and callbacks to handle media messages
        """
        self.image_sender = media_sender.ImageSender(interface_layer)
        self.video_sender = media_sender.VideoSender(interface_layer)
        self.url_print_sender = media_sender.UrlPrintSender(interface_layer)
        self.tts_sender = media_sender.EspeakTtsSender(interface_layer)
        self.routes = [
            ("https?:\/\/(?:[\w\-]+\.)+[a-z]{2,6}(?:\/[^\/#?]+)+\.(?:jpe?g|gif|png)($|\?[^\s]+$)", self.send_image),
            ("https?:\/\/(?:[\w\-]+\.)+[a-z]{2,6}(?:\/[^\/#?]+)+\.(?:mp4|webm)($|\?[^\s]+$)", self.send_video),
            ("https?:\/\/[^$]+$", self.send_url_print),
            ("^/t(ts)?\s(?P<tts_text>[^$]+)$", self.send_tts)
        ]

    def send_video(self, message, match):
        self.video_sender.send_by_url(jid=message.getFrom(), file_url=message.getBody())

    def send_image(self, message, match):
        self.image_sender.send_by_url(jid=message.getFrom(), file_url=message.getBody())

    def send_url_print(self, message, match):
        url = message.getBody()
        self.url_print_sender.send_by_url(jid=message.getFrom(), file_url=url)

    def send_tts(self, message, match):
        tts_text = match.group("tts_text")
        self.tts_sender.send(jid=message.getFrom(), text=tts_text)




from ..utils import media_sender
from yowsup.layers.protocol_messages.protocolentities.message_text import TextMessageProtocolEntity
import random


class SuperViews():
    def __init__(self, interface_layer):
        self.interface_layer = interface_layer
        self.url_print_sender = media_sender.UrlPrintSender(self.interface_layer)
        self.routes = [
            ("^/help", self.help),
            ("^/about", self.about),
            ("^/roll", self.roll),
            ("/(?P<evenOrOdd>even|odd)$", self.even_or_odd),
        ]

    def about(self, message=None, match=None, to=None):
        self.url_print_sender.send_by_url(message.getFrom(), "https://github.com/facerecog/wikimon", ABOUT_TEXT)

    def roll(self, message=None, match=None, to=None):
        return TextMessageProtocolEntity("[%d]" % random.randint(1, 6), to=message.getFrom())

    def even_or_odd(self, message=None, match=None, to=None):
        is_odd = len(match.group("evenOrOdd")) % 2
        num = random.randint(1, 10)
        if (is_odd and num % 2) or (not is_odd and not num % 2):
            return TextMessageProtocolEntity("[%d]\nYou win." % num, to=message.getFrom())
        else:
            return TextMessageProtocolEntity("[%d]\nYou lose!" % num, to=message.getFrom())

    def help(self, message=None, match=None, to=None):
        return TextMessageProtocolEntity(HELP_TEXT, to=message.getFrom())


HELP_TEXT = """ [HELP]
> Commands
/help - Show this message.
/about - About
/s(earch) - I'm lucky!
/i(mage) - I'm lucky with image!
/ping - Pong.
/echo - Echo.
/roll - Roll a dice.

> Chat
* - Type any message to begin chatting with it

> Downloads:
URL - (http://...) print screen.
"""

ABOUT_TEXT = """ [Facerecog Wikimon]
Wikimon is a Python script which creates an artificial intelligent digital pet that learns through mirroring.
https://github.com/facerecog/wikimon
"""



from threading import Timer
from yowsup.layers.protocol_groups.protocolentities.iq_groups_participants_add import AddParticipantsIqProtocolEntity
from yowsup.layers.protocol_groups.protocolentities.iq_groups_participants_remove import \
    RemoveParticipantsIqProtocolEntity
from yowsup.layers.protocol_messages.protocolentities import TextMessageProtocolEntity


class GroupAdminViews():
    """
        Group Administration Views

        If the bot have admin privileges on the group, it will be able to
        add/remove participants.
    """

    def __init__(self, interface_layer):
        self.interface_layer = interface_layer
        self.routes = [
            # Kick with optional time parameter in seconds
            ("^/kick\s(?P<phone_number>[0-9]{8,14})\s*(?P<time>[0-9]{2,3})?\s*$", self.kick),
            ("^/add\s(?P<phone_number>[0-9]{8,14})\s*$", self.add),
            ("^/ban\s(?P<phone_number>[0-9]{8,14})\s*$", self.ban),
        ]

    def kick(self, message, match):
        """
            Kicks user from group and add back in <time> seconds. Default time is 15
        """
        if self._is_authorized(message):
            jid_kick = self._get_jid(match.group("phone_number"))
            self._remove_user(message.getFrom(), jid_kick)
            kick_duration = int(match.group("time")) if match.group("time") else 15
            Timer(kick_duration, self._add_user, (message.getFrom(), jid_kick)).start()
            notify_message = "{ %s foi kickado por %d segundos }" % (match.group("phone_number"), kick_duration)
            self.interface_layer.toLower(TextMessageProtocolEntity(notify_message, to=message.getFrom()))

    def add(self, message, match):
        """
            Adds user from group and add back in <time> seconds. Default time is 15
        """
        if self._is_authorized(message):
            jid_kick = self._get_jid(match.group("phone_number"))
            self._add_user(message.getFrom(), jid_kick)

    def ban(self, message, match):
        """
            Remover user from group and not add back
        """
        if self._is_authorized(message):
            jid_kick = self._get_jid(match.group("phone_number"))
            self._remove_user(message.getFrom(), jid_kick)

    def _is_authorized(self, message):
        """
            Check if bot is authorized to add/remove users.
            For now just checks if the message is in group,
            # TO-DO: check if bot is in fact admin!
            # issue:
        """
        return message.isGroupMessage()

    def _get_jid(self, phone_number):
        """
            Build jid based on phone number.
        """
        return "%s@s.whatsapp.net" % phone_number

    def _remove_user(self, group_jid, user_jid):
        """
            Create Remove entity and sends
        """
        entity = RemoveParticipantsIqProtocolEntity(group_jid, [user_jid, ])
        self.interface_layer.toLower(entity)

    def _add_user(self, group_jid, user_jid):
        """
            Create Add entity and sends
        """
        entity = AddParticipantsIqProtocolEntity(group_jid, [user_jid, ])
        self.interface_layer.toLower(entity)



from distutils.core import setup
from setuptools.command.install import install
from setuptools import find_packages
import subprocess

class CustomInstall(install):
    def run(self):
        # custom stuff here

        command = "sudo apt-get install -y libjpeg-dev zlib1g-dev nodejs-legacy npm python-pip python-dev && sudo ln -s /usr/lib/x86_64-linux-gnu/libjpeg.so /usr/lib && sudo ln -s /usr/lib/x86_64-linux-gnu/libfreetype.so /usr/lib && sudo ln -s /usr/lib/x86_64-linux-gnu/libz.so /usr/lib && sudo npm -g install pageres-cli && sudo apt-get install -y espeak && sudo npm cache clean -f && sudo npm install -g n && sudo n stable && sudo ln -sf /usr/local/n/versions/node/v5.10.0/bin/node /usr/bin/node"
        subprocess.Popen(command).wait()
        install.run(self)

setup(
        name = 'wikimon_bot',
        packages = find_packages(), # this must be the same as the name above
        version = '0.0.7',
        description = 'Wikimon is a Python script which creates an artificial intelligent digital pet that learns through mirroring.',
        author = 'FaceRecog Pte Ltd',
        author_email = 'muhd.amrullah@facerecog.asia',
        url = 'http://facerecog.github.io/wikimon/', # use the URL to the github repo
        download_url = 'https://github.com/facerecog/wikimon/tarball/master',
        keywords = ['bot', 'chat'],
        classifiers = [],
        install_requires=['pytube', 'requests', 'python-Levenshtein', 'flask', 'oauthlib', 'python-axolotl', 'yowsup2'],
        scripts=['wikimon.py'],
        cmdclass={'install': CustomInstall},
        )





"""
    The Media Sender Utilities

    The MediaSender superclass handles the download, upload to whatsapp server and delivery to the receipt
    The subclasses VideoSender, ImageSender and YoutubeSender extends it to configure media type.
"""
from yowsup.layers.protocol_media.mediauploader import MediaUploader
from yowsup.layers.protocol_media.protocolentities.iq_requestupload import RequestUploadIqProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable import \
    DownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_image import \
    ImageDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_video import \
    VideoDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_audio import \
    AudioDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_messages.protocolentities import TextMessageProtocolEntity

import subprocess
import time
import os
import logging
import requests
import shutil
import hashlib
import re
from .. import config
from pytube import YouTube


class MediaSender():
    """
        This is a superclass that does the job of download/upload a media type.
        The classes bellow extends it and are used by the views.
    """

    def __init__(self, interface_layer, storage_path=config.media_storage_path):
        """
            The construction method receives the interface_layer (RouteLayer), so it can access the protocol methods
            to upload and send the media files
        """
        self.interface_layer = interface_layer
        self.storage_path = storage_path
        self.file_extension_regex = re.compile("^.*\.([0-9a-z]+)(?:[\?\/][^\s]*)?$")
        self.MEDIA_TYPE = None

    def send_by_url(self, jid, file_url, caption=None):
        """ Downloads and send a file_url """
        try:
            # self.interface_layer.toLower(TextMessageProtocolEntity("{...}", to=jid))
            file_path = self._download_file(file_url)
            self.send_by_path(jid, file_path, caption)
        except Exception as e:
            logging.exception(e)
            self._on_error(jid)

    def send_by_path(self, jid, path, caption=None):
        """
            Send a file by its absolute path.

            Creates a RequestUpload entity, that will verify if the media has already been uploaded.
            Then calls the _on_upload_result.
        """
        entity = RequestUploadIqProtocolEntity(self.MEDIA_TYPE, filePath=path)
        success_callback = lambda successEntity, originalEntity: self._on_upload_result(jid, path, successEntity,
                                                                                        originalEntity, caption)
        err_callback = lambda errorEntity, originalEntity: self._on_error(jid)
        self.interface_layer._sendIq(entity, success_callback, err_callback)

    def _download_file(self, file_url):
        """
            This method check for duplicate file before downloading,
            If not downloaded, download it, saves locally and returns the path
        """
        file_path = self._build_file_path(file_url)
        if not os.path.isfile(file_path):
            response = requests.get(file_url, stream=True)
            with open(file_path, 'wb') as out_file:
                shutil.copyfileobj(response.raw, out_file)
            del response
        return file_path

    def _on_upload_result(self, jid, file_path, upload_result, requestUploadIqProtocolEntity, caption=None):
        """
            If the file has never been uploaded, will be uploaded and then call the _do_send_file
        """
        if upload_result.isDuplicate():
            self._do_send_file(file_path, upload_result.getUrl(), jid, upload_result.getIp(), caption)
        else:
            callback = lambda file_path, jid, url: self._do_send_file(file_path, url, jid, upload_result.getIp(),
                                                                      caption)
            mediaUploader = MediaUploader(jid, self.interface_layer.getOwnJid(), file_path,
                                          upload_result.getUrl(),
                                          upload_result.getResumeOffset(),
                                          callback, self._on_error, self._on_upload_progress, async=True)
            mediaUploader.start()

    def _do_send_file(self, file_path, url, to, ip=None, caption=None):
        """
            Now the media file has been uploaded and the whatsapp server returns a media_path.
            The media_path is then sent to the receipt.
        """
        entity = None
        if self.MEDIA_TYPE == DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_VIDEO:
            entity = VideoDownloadableMediaMessageProtocolEntity.fromFilePath(file_path, url, self.MEDIA_TYPE, ip, to)
        elif self.MEDIA_TYPE == DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_IMAGE:
            entity = ImageDownloadableMediaMessageProtocolEntity.fromFilePath(file_path, url, ip, to, caption=caption)
        elif self.MEDIA_TYPE == DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_AUDIO:
            entity = AudioDownloadableMediaMessageProtocolEntity.fromFilePath(file_path, url, ip, to)
        self.interface_layer.toLower(entity)

    def _on_upload_progress(self, filePath, jid, url, progress):
        if progress % 50 == 0:
            logging.info("[Upload progress]%s => %s, %d%% \r" % (os.path.basename(filePath), jid, progress))

    def _on_error(self, jid, *args, **kwargs):
        self.interface_layer.toLower(TextMessageProtocolEntity("{!}", to=jid))

    def _get_file_ext(self, url):
        return self.file_extension_regex.findall(url)[0]

    def _build_file_path(self, url):
        id = hashlib.md5(url).hexdigest()
        return ''.join([self.storage_path, id, ".", self._get_file_ext(url)])


class VideoSender(MediaSender):
    def __init__(self, interface_layer):
        MediaSender.__init__(self, interface_layer)
        self.MEDIA_TYPE = RequestUploadIqProtocolEntity.MEDIA_TYPE_VIDEO


class AudioSender(MediaSender):
    def __init__(self, interface_layer):
        MediaSender.__init__(self, interface_layer)
        self.MEDIA_TYPE = RequestUploadIqProtocolEntity.MEDIA_TYPE_VIDEO


class ImageSender(MediaSender):
    def __init__(self, interface_layer):
        MediaSender.__init__(self, interface_layer)
        self.MEDIA_TYPE = RequestUploadIqProtocolEntity.MEDIA_TYPE_IMAGE


class YoutubeSender(VideoSender):
    """
        Uses pytube to download youtube videos
    """

    def _download_file(self, video_id):
        file_path = self._build_file_path(video_id)
        if not os.path.isfile(file_path):
            yt = YouTube()
            yt.from_url("http://youtube.com/watch?v=" + video_id)
            video = yt.filter('mp4')[0]
            video.download(file_path)
        return file_path

    def _build_file_path(self, video_id):
        return ''.join([self.storage_path, video_id, ".mp4"])


class UrlPrintSender(ImageSender):
    """
        Uses wkhtmltoimage to printscreen a webpage
    """

    def _download_file(self, page_url):
        page_url = page_url.replace('"', "'")
        file_path = self._build_file_path(page_url)
        file_name = os.path.splitext(os.path.basename(file_path))[0]
        if not os.path.isfile(file_path):
            cmd = 'pageres "%s" 1024x2300 --crop  --filename=%s' % (page_url, file_name)
            p = subprocess.Popen(cmd, shell=True, cwd=self.storage_path)
            p.wait()
        return file_path

    def _build_file_path(self, page_url):
        id = hashlib.md5(page_url).hexdigest()
        return ''.join([self.storage_path, id, str(int(time.time()))[:-2], ".png"])


class EspeakTtsSender(AudioSender):
    """
        Uses espeak to text to speach
    """

    def send(self, jid, text, lang='en'):
        text = text.replace("'", '"')
        try:
            file_path = self.tts_record(text, lang)
            self.send_by_path(jid, file_path)
        except Exception as e:
            logging.exception(e)
            self._on_error(jid)

    def tts_record(self, text, lang='en'):
        file_path = self._build_file_path(text)
        cmd = "espeak -v%s -w %s '%s'" % (lang, file_path, text)
        subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).wait()
        return file_path

    def _build_file_path(self, text):
        id = hashlib.md5(text).hexdigest()
        return ''.join([self.storage_path, id, ".wav"])



import shelve


class SessionDB:
    """
        In real world applications, choosing a very efficient way to store
        sessions on databases is essential. Here it's ok.
        Shelve is pure simple python to create a disk persistent map.
        https://docs.python.org/2/library/shelve.html#example
    """

    def __init__(self, db_path, view_name):
        self.db_path = db_path
        self.view_name = view_name

    def get(self, key):
        _db = shelve.open(self.db_path)
        viewdb = _db.get(self.view_name, {})
        _db.close()
        return viewdb.get(key)

    def set(self, key, value):
        _db = shelve.open(self.db_path)
        try:
            viewdb = _db.get(self.view_name, {})
            viewdb[key] = value
            _db[self.view_name] = viewdb
        finally:
            _db.close()



from yowsup.layers.protocol_messages.protocolentities import TextMessageProtocolEntity
from req import getResponse
import re

def echo(message, match):
    print message.getBody()
    return TextMessageProtocolEntity("Echo: %s" % match.group("echo_message"), to=message.getFrom())


def ping(message, match):
    return TextMessageProtocolEntity("Pong!", to=message.getFrom())

def superbot(message, match):
    clean_bot_response = re.sub(r'(.)\1{3,}', '', message.getBody())
    bot_response = getResponse(clean_bot_response)
    string_bot_response = str(bot_response)
    string_bot_response = re.sub(r'(.)\1{3,}', '', string_bot_response)
    return TextMessageProtocolEntity(string_bot_response, to=message.getFrom())
